/**
 * BSDialog5
 * @version v0.1.1.063 (2023/07/275 1713)
 */
var BSDialog5 = {
	Create: async function (id, title, url, size) {
		var a = this;

		a.pfx = "bsdia5_";
		a.body = document.getElementsByTagName("body")[0];

		a.addBackdrop();
		a.addModal(id, title, size);

		if (a.isNullOrWhitespace(url)) {
			a.UpdateBody(id, "");
		} else {
			await a.UpdateBodyRemote(id, url);
		}
	},
	Clear: function () {
		this.body.querySelectorAll(".modal").forEach(function(e) {
			e.parentNode.removeChild(e);
		});

		this.removeBackdrop();
	},
	Close: function (id) {
		let modal = this.Find(id);
		if (modal === null) {
		 	return;
		}

		modal.Modal.forEach(function(e) {
			e.parentNode.removeChild(e);
		});

		modal = this.Find(id);
		if (modal !== null) {
			return;
		}

		this.removeBackdrop();
	},
	UpdateTitle: function (id, title) {
		let modal = this.Find(id);
		if (modal === null) {
		 	return;
		}

		modal.Title.forEach(function(e) {
			e.innerHTML = title;
		});
	},
	UpdateSize: function (id, size) {
		let modal = this.Find(id);
		if (modal === null) {
		 	return;
		}

		modal.Modal.forEach(function(e) {
			e.classList.remove("modal-sm");
			e.classList.remove("modal-md");
			e.classList.remove("modal-lg");
			e.classList.remove("modal-xl");
			e.classList.add("modal-" + size);
		});
	},
	UpdateBody: function (id, content) {
		var a = this;

		let modal = a.Find(id);
		if (modal === null) {
		 	return;
		}

		modal.Body.forEach(function(e) {
			a.html(e, content);
		});
	},
	UpdateBodyRemote: async function (id, url) {
		var a = this;

		if (!a.Exists(id)) {
		 	return;
		}

		await fetch(url, {
			cache: 'no-cache',
			credentials: 'same-origin'
		}).then(data => data.text()).then(data => {
			a.UpdateBody(id, data);
		}).catch((error) => {
			a.UpdateBody(id, "Error: " + error);
		});
	},
	Exists: function (id) {
		return (this.Find(id) !== null);
	},
	Find: function (id) {
		let modal = this.body.querySelectorAll("#" + this.pfx + id + ".modal");
		if (!modal) {
			return null;
		}

		if (modal.length <= 0) {
			return null;
		}

		return {
			Title: modal[0].querySelectorAll(".modal-title"),
			Body: modal[0].querySelectorAll(".modal-body"),
			Footer: modal[0].querySelectorAll(".modal-footer"),
			Close: modal[0].querySelectorAll("[data-bs-dismiss='modal']"),
			Modal: modal
		};
	},
	ShowToast: function (id, title, message, size) {
		this.Create(id, title, null, size);
		this.UpdateBody(id, message);
	},
	addBackdrop: function () {
		let a = this;

		// don't allow duplicates
		if (a.body.querySelectorAll(".modal-backdrop").length > 0) {
			return;
		}

		a.appendHtml(a.body, '<div class="modal-backdrop fade show"></div>');

		// lock background
		a.body.classList.add("modal-open");
		a.body.style.overflow = "hidden";

		// close
		let backdrop = a.body.querySelectorAll(".modal-backdrop")[0];
		backdrop.addEventListener("click", function(e){
			e.stopPropagation();
			e.preventDefault();

			a.Clear();
		});
	},
	addModal: function (id, title, size) {
		var a = this;

		// don't allow duplicates
		let modal = a.Find(id);
		if (modal !== null) {
		 	return;
		}

		let html = "";
		html += '<div class="modal modal-' + size + ' fade show d-block" id="' + a.pfx + id + '" tabindex="-1" aria-modal="true" role="dialog">';
		html += '  <div class="modal-dialog">';
		html += '    <div class="modal-content">';
		html += '      <div class="modal-header">';
		html += '        <span class="modal-title fs-5">' + title + '</span>';
		html += '        <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>';
		html += '      </div>';
		html += '      <div class="modal-body">';

		html += '<div class="row mt-3 pb-3">';
		html += '  <div class="col-12 text-center">';
		html += '    <div class="spinner-border">';
		html += '	     <span class="visually-hidden">Loading...</span>';
		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		html += '      </div>';
		html += '      <div class="modal-footer">';
		html += '        <button type="button" class="btn btn-outline-secondary" data-bs-dismiss="modal">Close</button>';
		html += '      </div>';
		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		a.appendHtml(a.body, html);

		modal = a.Find(id);
		if (modal === null) {
			return;
	 	}

		// modal.addEventListener("click", function(e){
		// 	e.stopPropagation();
		// 	e.preventDefault();

		// 	a.Close(id);
		// });

		modal.Close.forEach(function(e){
			e.addEventListener("click", function(e){
				e.stopPropagation();
				e.preventDefault();

		 		a.Close(id);
		 	});
		});
	},
	removeBackdrop: function () {
		if (this.body.querySelectorAll(".modal-backdrop").length <= 0) {
			return;
		}

		let backdrop = this.body.querySelectorAll(".modal-backdrop")[0];
		backdrop.parentNode.removeChild(backdrop);

		// unlock background
		this.body.classList.remove("modal-open");
		this.body.style.overflow = null;
	},
	appendHtml: function (el, html) {
		let node = document.createElement('template');
		node.innerHTML = html;

		node = node.content.firstChild;

		el.appendChild(node);
	},
	html: function (el, newHtml) {
		/// todo: replace with pure JS
		if (jQuery) {
			jQuery(el).html(newHtml);
		} else {
			el.innerHTML = newHtml;
		}
	},
	isNullOrWhitespace: function(e) {
		if (typeof (e) == "undefined") {
			return true;
		}

		if (e == null) {
			return true;
		}

		if (e == false) {
			return true;
		}

		return (e.trim().length <= 0);
	}
};